;
; load.asm - LOAD kernal extension handler
;
; Written by
;  Kajtar Zsolt <soci@c64.rulez.org>
;
;    This file is part of IDEDOS the IDE64 disk operating system
;    See README for copyright notice.
;
;    This program is free software; you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation; either version 2 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this program; if not, write to the Free Software
;    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;
;**************************************************
; Load (A=verify, X,Y=load address)
;
; Used registers: A X Y (X,Y=next byte after end)
; Zeropage: $c3,$c4=X and Y register before call
; Zeropage: $ae,$af=next byte after end
; Status: C
; C=1:
; A=0 break
; A=4 file not found
; A=5 device not present
; A=8 missing filename
; A=9 illegal device number
; A=16 out of memory
; Status response:
; ST=$00 ok (only if run/stop was pressed)
; ST=$10 load/verify error (only if run/stop was pressed)
; ST=$40 load ok
; ST=$42 open error
; ST=$50 verify error
; ST=$52 load/verify error because of readerror (open error, output file, device...)
; ST=$80 device not present
;
; Notes:
; Secondary address 1..255 then load address from file is used
;**************************************************
;--------------------------------------
; LOAD
;
;		stx $c3
;		sty $c4
;		jmp ($0330)
;--------------------------------------
newload		.proc
		sta VerifyFlag		;Store verify flag (0=load, else verify)
		#stz_a Status		;Clear status
		stx $ae
		sty $af			;Store loading address ($c3,$c4 is the same...)
                
                .if DEBUGIO=1
                lda debug_io_exp
                beq kd
                #lda0
                sta debug_io_exp
                #psr $f4a5
                php
                lda VerifyFlag
                pha
                #phx_a
                #phy_a
                lda #3
                jmp monitor

kd		lda #1
		sta debug_io_exp
                .fi

		lda ActualDev
                beq kernal_load.ill
		#devicehandler hard_load,pclink_load2,exit_illegal_device,kernal_load
		.pend

kernal_load	.proc
		tya
		beq ill
		cmp #31
		bcs ill
		cmp #3
		beq ill
		bcs ok
		#psr $f4b6
		tya
		jmp exit2basic		;Normal kernal rutin
ill		jmp exit_illegal_device

ok
		ldy ActualFNLength
		beq exit_missing_filename
		sta load_p_exp
		pha
		#OP_scpu01up
		jsr OP_load_common

		lda ActualSecAddr
		sta load_limit_exp
		lda #$60
		sta ActualSecAddr

		jsr OP_cfg16up
		pla
		tay
		.if SERIAL_HANG_FIX=1
		jsr test_hang
		bcc np
		.fi
		jsr ieclisten		;listen
		lda #$f0
		jsr iecsecond		;second
		lda Status
		bpl oe
np		jsr OP_cfg16down
		jsr OP_restore_zeropage
		#OP_scpu01down
		jmp exit_device_not_present
oe
		ldx #0
cy		#sta_cfg_open
		lda path_exp,x
		#sta_cfg_16
		beq oy
		jsr iecout		;ciout
		inx
		bne cy
oy
		jsr iecunlisten		;unlsn

		tya
		jsr iectalk		;talk
		lda #$60
		jsr iectksa		;tksa
		jsr iecin
		tax
		lda Status
		and #2
		beq oe2
		jsr OP_cfg16down
		jsr OP_restore_zeropage
		#OP_scpu01down
		jmp exit_file_not_found

oe2
		.if (JIFFYDOS_ACCEL=1) & (SUPERCPU=0)
		jsr iecin
		pha
		bit $a3
		bvc wr
		#sta_cfg_open
		lda path_exp
		#sta_cfg_16
		cmp #"$"
		beq wr
		jsr iecuntalk
		tya
		jsr iectalk
		lda #$61
		jsr iectksa
		asl $a3			;fastload!
wr		pla
		.else
		jsr iecin
		.fi
		tay
		jsr OP_cfg16down

		lda load_limit_exp
		beq up
		stx $ae
		sty $af
up

		lda #<$0333
		cmp $ae
		lda #>$0333
		sbc $af
		bcc above

		ldy #1			;out of memory
		gne ex
above
		jsr OP_load_loading	;LOADING $xxxx-...

		#lda0
		ldx $af
		cpx #$d8
		bcc fg
		cpx #$dc
		bcs fg
		lda #$dc
fg		sta load_top_exp

		#OP_scpu01down

		.if (JIFFYDOS_ACCEL=1) & (SUPERCPU=0)
		bit $a3
		bpl oldl

		sei
		lda $d015
		pha
		jsr jiffy_f0d8		;turn off sprites
		ldy $ae
		#lda0
		sta $ae
nw		.byte SKIP2
		lda #1
		sta next_sector		;first
		#sta_cfg_open
		jsr OP_cfg16up
		jsr $f6bc		;check stop
		php
		jsr OP_cfg16down
		plp
		bpl stpp
		lda start_zeropage
		cmp #$60		;rts?
		beq stpp
		lda $d011
		and #$17
                cmp #$10
                bcs lc
		lda #$e1
lc      	adc #$1f-1
		sta next_sector+1
		lda $dd00
		and #7
		sta next_sector+2
		sta $dd00
		ora #$20
		tax
		bit $dd00
		bvc *-3
		bpl msc
		ldx #$64
kkw		bit $dd00
		bvc eof
		dex
		bne kkw
		lda #$42
		.byte SKIP2
eof		lda #$40
		ora Status
		sta Status
		clc
		.byte SKIP1
stpp		sec
		pla
		sta $d015
		sty $ae
		ldy #0
		bcc ex
		gcs sto

mmc		#sta_cfg_16
		sec
mc		lda $d012
		sbc next_sector+1
		bcc mcx
		and #7
		beq mc
mcx		lda next_sector+2
		stx $dd00
		bit $dd00
		bvc nw
		nop
		sta $dd00
		ora $dd00
		lsr
		lsr
		nop
		ora $dd00
		lsr
		lsr
		eor next_sector+2
		eor $dd00
		lsr
		lsr
		eor next_sector+2
		eor $dd00
		cpx #128
		bcc va
		sta ($ae),y
vav		iny
		nop			;ntsc ;(
		bne mc                
		#sta_cfg_open
		inc $af
		lda $af
		cmp load_top_exp
		bne bc
		dec $af
                txa			;skip
                and #$7f
                tax
		lda #$60		;rts
		sta start_zeropage
bc		cmp #$d0
		beq mc2
		cmp #$e0
		bne mmc
		geq mc2

va		jsr start_zeropage
		sec
		gcs vav

msc		bit $dd00
		bpl *-3
		lda next_sector		;first?
		bne mmce		;no

mc2		#phx_a
		#phy_a
		jsr load_install_write_verify_byte3
		#ply_a
		pla
                and #$7f
		ldx start_zeropage+3
		cpx #$91		;sta ($nn),y
		bne nbv
		ora #$80
nbv		sta current_path

mmce		ldx current_path
		jmp mmc

oldl
		.fi

loop2		jsr load_install_write_verify_byte3

		ldy #0
loop
		bit Status
		bvs ex
		jsr OP_cfg16up
		jsr iecin
		tax
		jsr OP_cfg16down
		txa
		jsr start_zeropage
		inc $ae
		bne loop
		jsr check_stop
		beq sto
		inc $af
		lda $af
		cmp load_top_exp
		beq out
		cmp #$d0
		beq loop2
		cmp #$e0
		bne loop
		geq loop2

sto		iny
out		iny
ex
		.if (JIFFYDOS_ACCEL=1) & (SUPERCPU=0)
		lda start_zeropage
		cmp #$60		;rts?
		bne noto
		lda load_top_exp
		sta $af
		ldy #0
		sty $ae
		iny			;out of memory!
noto
		.fi

		lda load_p_exp
		pha
		#OP_scpu01up
		jsr OP_cfg16up
		jsr iecuntalk		;untalk
		pla
		jsr ieclisten		;f648
		lda #$e0
		jsr iecsecond
		jsr iecunlisten
		jsr OP_cfg16down
		tya
		beq load_end
out2		jsr OP_restore_zeropage
		#OP_scpu01down
		dey
		sec
		beq outu
		#lda0
		jmp exit2basic		;stop pressed

outu		jmp exit_out_of_memory
		.pend
;-----------------------------------
; Install zeropage routine for kernal loader
;
load_install_write_verify_byte3 .proc
		lda $af
		cmp #$d0
		bcc u
		cmp #$e0
		bcs u
		lda VerifyFlag
		bne u
		lda load_top_exp
		bne u
		lda #$4c		;fake
		sta start_zeropage
                sta start_zeropage+3	;signal!
		lda #<write_byte_io_r3
		sta start_zeropage+1
		lda #>write_byte_io_r3
		sta start_zeropage+2
		rts

u		ldx #14
vsd		lda write_byte_r3,x
		ldy VerifyFlag
		beq cbd
		lda verify_byte_r3,x
cbd		sta start_zeropage,x
		dex
		bpl vsd
		rts
		.pend
;-----------------------------------
; Write a byte (for load)
;
write_byte_r3	.proc
		.logical start_zeropage
		sta cfg_stnd		;3
		sta ($ae),y		;2
		sta cfg_open		;3
		rts			;1
		.here
		.pend
;-----------------------------------
; Verify a byte (for load)
;
verify_byte_r3	.proc
		.logical start_zeropage
		sta cfg_stnd		;3
		cmp ($ae),y		;2 5
		sta cfg_open		;3
		beq *+6			;2 5 15
		lda #$10		;2
		sta Status		;2
		rts			;1
		.here
		.pend
;-----------------------------------
; loading bytes under I/O
;
write_byte_io_r3 .proc
		#sta_cfg_16
		php
		pha
		lda #2
		sei
		sta $1
		pla
		sta ($ae),y
		lda #6
		plp
		sta $1
		jmp OP_cfg16down
		.pend
;--------------------------------------
; LOAD from PCLink
;--------------------------------------
pclink_load2	.proc                
                lda ActualFNLength
		beq exit_missing_filename

		#OP_scpu01up
		jsr OP_load_common

		jsr load_install_write_verify_byte3

		jsr pclink_load
		bcc load_end
                dey
                bmi hard_load.outmem
                tya
                beq hard_load.brek	;stop
                gne hard_load.fnf
		.pend
;--------------------------------------
; LOAD from IDE
;--------------------------------------
hard_load	.proc
		lda ActualFNLength
		beq exit_missing_filename
		#OP_scpu01up
		jsr OP_load_common

		ldy #0
		tsx
fg		cpx #$03+52		;enough stack space?
		bcs ook
		pla
		sta sp_exp,x		;no, let's backup
		iny
		inx
		gne fg
ook		#phy_a

		jsr XIT_load
		sta help_x_exp
		#sty_l_a help_y_exp

		#ply_a
		beq kb
		ldx #$03+52
nk		dex
		lda sp_exp,x		;restore stack
		pha
		dey
		bne nk
kb
		#ldy_l_a help_y_exp
		lda help_x_exp
		bcc nerr
		#ldatax current_drive_exp
		lda status_pos_exp,x
		cmp #$80+39
		beq fnf
		cmp #$80+62
		beq fnf
		lda #$52		;Error... (load error)
		sta Status
nerr		tax
		dex
		beq brek
		dex
		bne load_end

outmem		jsr OP_restore_zeropage
		#OP_scpu01down
		jmp exit_out_of_memory

brek		jsr OP_restore_zeropage
		#OP_scpu01down
		#lda0
		sec
		jmp exit2basic

fnf		jsr OP_restore_zeropage
		#OP_scpu01down
		jmp exit_file_not_found

load_end	.proc
		jsr OP_restore_zeropage
		tya
		clc
		adc $ae
		sta $ae
		bcc *+4
		inc $af
		lda load_autostart2_exp	;autostart?
		beq tp
		bmi jmpp
		ldx #4
		stx 198
ne		lda run-1,x
		sta 630,x
		dex
		bne ne
		geq tp

jmpp		lda load_autostart_start_exp
		sec
		sbc #1
		tax
		lda load_autostart_start_exp+1
		sbc #0
		pha
		#phx_a

tp		#lda0
		sta load_autostart_exp	;no autostart
		lda Status
		ora #%01000000
		sta Status
		and #%10101111
		bne *+5
		jsr OP_load_loading_end
		#OP_scpu01down
		ldx $ae
		ldy $af
		clc
		jmp exit2basic
run		.text "R","U"+128,":",13

XIT_load	.proc

		#xit_init

		jsr check_drive		;check drive is alive

		jsr XIT_init_drive	;-=> error

		lda path_exp
		cmp #"$"
		beq XIT_load_dir	;LOAD"$"?
		gne XIT_load_file
		.pend
;--------------------------------------
; LOAD directory
;
XIT_load_dir	.proc
		jsr load_install_write_verify_byte

		ldx ActualSecAddr	;Load to specifyed address?
		beq jkl
		lda #1
		sta $ae
		lda #4			;Load address from file ($0401)
		sta $af
jkl
		lda $ae
		sta start_zeropage+5
		cmp #<$0334
		lda $af
		sta start_zeropage+6
		sbc #>$0334
		bcc mem

		lda #1
		jsr XIT_dir_first	;Init dir stuff

		jsr OP_load_loading	;LOADING $xxxx-...
load_dirlp	jsr check_stop
		beq stop

		jsr XIT_dirlp2		;-=> error
		lda direntry_first_exp
		bpl load_dirlp

		ldy start_zeropage+5
		#lda0
		sta $ae
		.byte SKIP2
stop		lda #1			;-=> break
		.byte SKIP2
mem		lda #2			;-=> out of memory
		ldx start_zeropage+6
		stx $af
		clc
		rts			;-=> ok
		.pend
;-----------------------------------
; LOAD file
;
XIT_load_file	.proc
		lda #RESOLV_LOAD_MODE
		jsr OP_XIT_resolv_path_part;-=> file not found

		lda resolv_anything_exp
		sec
		beq LWOP_reporterror_62_xit;LOAD CHR$(0),12

                jsr OP_get_current_drive_type
                and #FS_MASK
                cmp #FS_CFS
                beq load_cfs
                sec
                gcs LWOP_reporterror_62_xit;only cfs is supported now
load_cfs
		lda #ram_exp_buffer0	;Load tree sector
		jsr XIT_read_a_sector	;-=> error

		lda direntry_f_size_exp+2
		ora direntry_f_size_exp+3;file bigger than 65535 bytes?
		beq notoobig
		lda #255
		sta direntry_f_size_exp	;65535 will also cause OUT OF MEMORY...
		sta direntry_f_size_exp+1
notoobig

		lda ActualSecAddr
		bne abs
					;load to (X/Y)
		sta load_p_exp		;from sector 0

		jsr load_calc_needed
		bcs vwv
		lda #2
		rts			;-=> out of memory
vwv
		lda load_size_exp
		pha
		adc #2-1
		sta load_size_exp	;2 more bytes for first lincalc
		php
		bcc ry
		#inc_l_a load_size_exp+1
ry

		ldy #ram_exp_buffer0
		jsr linearaccess

		plp
		bcc ryy
		#dec_l_a load_size_exp+1
ryy		pla
		sta load_size_exp
		#jmps rel1

abs		lda #1
		sta load_sectors_exp	;1 sector loaded
		sta load_p_exp		;from sector 1

		#cnext ram_exp_buffer0_exp , next_sector;first sector address

rel1		lda next_sector
		ora next_sector+1
		ora next_sector+2
		ora next_sector+3
                bne mcx	       		;hole in file?
                tax
nc              sta ram_exp_buffer0b_exp,x
                sta ram_exp_buffer0b_exp+$100,x
                inx
                bne nc
                geq bd

mcx		jsr XIT_upload_next_sector;-=> error
		#ldy_l_a load_sectors_exp;1 sector
		jsr OP_read
		jsr LWOP_reporterror_read_xit;-=> error
		ldx #ram_exp_buffer0b
		jsr read_buffer_cfs

bd		ldx ActualSecAddr	;Load to (X/Y)?
		beq rel2
		lda load_address_exp	;no
		sta $ae
		lda load_address_exp+1
		sta $af

		jsr load_calc_needed
		bcs rel2
		jsr ata_pi_nomore_data
		jsr LWOP_reporterror_read_xit;-=> error
		lda #2
		rts			;-=> out of memory

rel2
		jsr OP_load_loading

		lda VerifyFlag
		beq loadc

		ldx #51
vsd2		lda ver_lot_cfs_r,x
		pha
		dex
		bpl vsd2
		gmi nl2

loadc		#psr back+1
		ldx #load_lot_cfs_re-load_lot_cfs_r-1
vsd		lda load_lot_cfs_r,x
		pha
		dex
		bpl vsd

nl2

		lda #2
		sta pom1
		lda #ram_exp_buffer0b
		sta pom2

		lda load_size_exp	;file smaller than 510 bytes?
		sec
		sbc #<510
		tax
		lda load_size_exp+1
		sbc #>510
		ldy #2
		bcc lessthan512be
		sta load_size_exp+1
		#stx_l_a load_size_exp

		jsr install_verify

		#OP_scpu01down
		jsr OP_cfg16up
		#sta_cfg_open

		.if SUPERCPU=1
		lda #1
		sta pom3
		.fi

		ldy #0
		ldx #254
		jsr load_verify_part	;copy 254 bytes
		tya
		clc
		adc $ae			;adjust destination address
		sta $ae
		bcc *+4
		inc $af

		#lda0
		sta pom1		;adjust buffer address
		inc pom2

		tay
		tax
		jsr load_verify_part	;copy 256 bytes
		jsr OP_cfg16down
		#OP_scpu01up

		clc

		tsx
		lda $ae
		sta load_lot_cfs_r.at1+1,x

		ldy VerifyFlag
		beq lll

		sta load_lot_cfs_r.at3+1+4,x
		adc #$80
		sta load_lot_cfs_r.at2+1+2,x
		sta load_lot_cfs_r.at4+1+6,x
		lda $af
		sta load_lot_cfs_r.at1+2,x
		adc #255
		sta load_lot_cfs_r.at2+2+2,x
		clc
		gcc be

lll		sta load_lot_cfs_r.at3+1,x
		adc #$80
		sta load_lot_cfs_r.at2+1,x
		sta load_lot_cfs_r.at4+1,x
		lda $af
		sta load_lot_cfs_r.at1+2,x
		adc #255
		sta load_lot_cfs_r.at2+2,x
		clc
		gcc be


vback		iny
		dey
		bmi jo
		lda #$10
		sta Status
		bcc ly
		lda data_g		;skip the rest
ly		iny
		bmi jo
		lda data_g		;skip the rest
		lda data_g		;skip the rest
		#jmps ly

tnt
		ldy #ram_exp_buffer0
		jsr linearaccess

		jsr XIT_upload_next_sector;-=> error

		#ldy_l_a load_sectors_exp;1 sector
		jsr OP_read
		bcs handle_error

		tsx
		lda status_g
		lda low_reg
read_loop	and #%00001000
		beq handle_error2	;DRQ=1?

		lda load_size_exp+1
		sec
		sbc #2
		bcc lessthan512
		sta load_size_exp+1

		ldy #0
		lda VerifyFlag
		bne we
		lda load_lot_cfs_r.at1+2,x
		cmp #$ce
		bcc we
		cmp #$e0
		bcc load_lot_io_cfs_r
we
		sec
		#OP_scpu01down
		lda #1
		pha
		#phx_a
		rts

back		#OP_scpu01up

jo		sec
be		tsx
		lda load_lot_cfs_r.at1+2,x
		adc #1
		sta load_lot_cfs_r.at1+2,x
		#adc1

		ldy VerifyFlag
		beq llo
		sta load_lot_cfs_r.at3+2+4,x
		lda load_lot_cfs_r.at2+2+2,x
		adc #2
		sta load_lot_cfs_r.at2+2+2,x
		#adc1
		sta load_lot_cfs_r.at4+2+6,x
		jmp llo2

llo		sta load_lot_cfs_r.at3+2,x
		lda load_lot_cfs_r.at2+2,x
		adc #2
		sta load_lot_cfs_r.at2+2,x
		#adc1
		sta load_lot_cfs_r.at4+2,x

llo2
		lda status_g
		lda low_reg
		bpl okbsy
		jsr wait_busy
		bcs handle_error
		lda status_g
		lda low_reg
		tsx
okbsy
		#dec_l load_sectors_exp
		bne read_loop

		and #%00001000
		bne handle_error2	;DRQ=0?

		lda load_size_exp
		ora load_size_exp+1
		bne tnt
		tay
		geq eof

handle_error2	sec
		lda #EUNEXP
handle_error	jmp LWOP_reporterror_read_xit

lessthan512	ldx #ram_exp_buffer0b
		jsr read_buffer_cfs

		#stz_a pom1
		lda #ram_exp_buffer0b
		sta pom2
		tsx
		lda load_lot_cfs_r.at1+2,x
		sta $af			;$ae is correct...
lessthan512be
		jsr ata_pi_nomore_data
		bcs handle_error

		.if SUPERCPU=1
		lda #1
		sta pom3
		.fi

		jsr install_verify

		#OP_scpu01down
		jsr OP_cfg16up
		#sta_cfg_open

		ldy #0
		lda load_size_exp+1
		beq lesst256
		ldx #0
		jsr load_verify_part
		inc $af
		inc pom2

lesst256	#ldatax load_size_exp
		beq tg
		jsr load_verify_part
tg
		jsr OP_cfg16down
		#OP_scpu01up

eof		#xit_exit

		clc
		lda load_overflow_exp
		bmi nov
		lda load_top_exp
		sta $af
		#stz_a $ae
		lda #2			;out of memory
		.byte SKIP2
nov		lda #0
		rts			;-=> ok

load_lot_io_cfs_r
		lda load_lot_cfs_r.at4+2,x
		sta pom5
		lda load_lot_cfs_r.at3+2,x
		sta pom7
		lda load_lot_cfs_r.at2+2,x
		sta pom9
		lda load_lot_cfs_r.at1+2,x
		sta pom11
		lda load_lot_cfs_r.at4+1,x
		sta pom4
		sta pom8
		lda load_lot_cfs_r.at3+1,x
		sta pom6
		sta pom10
		jsr OP_cfg16up
		php
		pla
		and #%00000100
		beq lp			;sei?

lp2		ldx #2
		lda data_g
		lda low_reg
		pha
		lda hig_reg
		pha
		lda data_g
		lda low_reg
		sta pom12
		lda hig_reg
		stx $1
		sta (pom4),y
		lda pom12
		sta (pom6),y
		pla
		sta (pom8),y
		pla
		sta (pom10),y
		ldx #6
		stx $1

		iny
		bpl lp2
		gmi lw

lp		ldx #2
		lda data_g
		lda low_reg
		pha
		lda hig_reg
		pha
		lda data_g
		lda low_reg
		sta pom12
		lda hig_reg
		sei
		stx $1
		sta (pom4),y
		lda pom12
		sta (pom6),y
		pla
		sta (pom8),y
		pla
		sta (pom10),y
		ldx #6
		cli
		stx $1

		iny
		bpl lp

lw		jsr OP_cfg16down
		jmp jo
		.pend
;-----------------------------------
; Load/verify part of the buffer
;
load_verify_part .proc
		lda VerifyFlag
		bne ry5c
		lda $af
		cmp #$cf
		bcc ry5
		cmp #$e0
		bcs ry5
ry4		#lda_zp_y pom1
		#sta_cfg_16
		php
		pha
		lda #2
		sei
		sta $1
		pla
		sta ($ae),y
		lda #6
		plp
		sta $1
		#sta_cfg_open
		iny
		dex
		bne ry4
		rts

ry5		#lda_zp_y pom1
		#sta_cfg_16
		sta ($ae),y
		#sta_cfg_open
		iny
		dex
		bne ry5
		rts

ry5c		#lda_zp_y pom1
		jsr start_zeropage
		beq dcc
		lda #$10
		sta Status
dcc		iny
		dex
		bne ry5c
		rts
		.pend
;-----------------------------------
; Installs small verify routine for load_verify_part
;
install_verify	.proc
		ldy VerifyFlag
		beq nverf
		ldx #8
vsd2		lda verify_1_cfs_r,x
		beq n2
		sta start_zeropage,x
n2		dex
		bpl vsd2
nverf		rts
		.pend
;-----------------------------------
; CFS verifying 1 byte normal Z=0 if no match (for verify_part)
;
verify_1_cfs_r	.proc
		.logical start_zeropage
		sta cfg_stnd		;3
		eor ($ae),y		;5
		sta cfg_open		;8
		rts			;9
		.here
		.pend
;-----------------------------------
; Calculate number of bytes to load to load_size_exp
; C=1 if loading below $0334
;
load_calc_needed .proc
		lda $ae
		cmp #<$0334
		lda $af
		sbc #>$0334
		bcs above
		rts

above		ldy #>$0000		;memory end

		lda direntry_f_size_exp
		sbc #2
		sta direntry_f_size_exp
		lda direntry_f_size_exp+1
		sbc #0
		bcs nt			;file shorter than 2 bytes???
		tya
		sta direntry_f_size_exp
nt		sta direntry_f_size_exp+1

		ldx $af			;loading limits ($10000/$dc00)
		cpx #>$d800
		bcc nl1
		cpx #>$dc00
		bcs nl1
		ldy #>$dc00		;memory end
nl1
		#lda0
		sec
		sbc $ae
		tax
		tya			;Calculate space until limit
		sta load_top_exp
		sbc $af
		tay

		.if SUPERCPU=1
		txa
		cmp direntry_f_size_exp
		tya
		.else
		cpx direntry_f_size_exp
		.fi
		sbc direntry_f_size_exp+1
		tya
		bcc ok
		#ldatax direntry_f_size_exp;space is bigger than file...
		lda direntry_f_size_exp+1;no overflow

ok		sta load_size_exp+1
		#stx_l_a load_size_exp

		.if SUPERCPU=1
		ror
		sta load_overflow_exp
		.else
		ror load_overflow_exp	;if overflow<128 than file was bigger than available space
		.fi
		sec
		rts
		.pend
;-----------------------------------
; CFS load 512 bytes normal
;
load_lot_cfs_r	.proc
		.logical $101
		sta cfg_stnd		;3
lp		lda data_g		;6

		lda low_reg		;9
at1		sta $0000,y		;12

		lda hig_reg		;15
at2		sta $0000,y		;18

		lda data_g		;21

		lda low_reg		;24
at3		sta $0000,y		;27

		lda hig_reg		;30
at4		sta $0000,y		;33

		iny			;34

		bpl lp			;36
		sta cfg_open		;39
		.byte $4c		;40
		.here
		.pend
load_lot_cfs_re
;-----------------------------------
; CFS verifying 512 bytes normal Z=0 if no match
;
ver_lot_cfs_r	.proc
		.logical $101
		sta cfg_stnd		;3
lp		lda data_g		;6

		lda low_reg		;9
at1		eor $0000,y		;12
		bne err			;14
		lda hig_reg		;17
at2		eor $0000,y		;20
		bne err			;22

		lda data_g		;25

		lda low_reg		;28
at3		eor $0000,y		;31
		bne err2		;33
		lda hig_reg		;36
at4		eor $0000,y		;39
		bne err2		;41

		iny			;42

		bpl lp			;44
		.byte $a9		;45
err2		clc			;46
err
		sta cfg_open		;49
		jmp XIT_load_file.vback	;52
		.here
		.pend



;-----------------------------------
; Install zeropage routine for directory stuff
;
load_install_write_verify_byte .proc
		ldx #19
vsd		lda write_byte_r,x
		ldy VerifyFlag
		beq cbd
		lda verify_byte_r,x
cbd		sta start_zeropage,x
		dex
		bpl vsd
		rts
		.pend
;-----------------------------------
; Write a byte (for directory)
;
write_byte_r	.proc
		.logical start_zeropage
		sta cfg_stnd		;3
lp		pla			;4
		sta $0000,y		;7
		dey			;8
		bpl lp			;10
		sta cfg_open		;13
		rts			;14
		.here
		.pend
;-----------------------------------
; Verify a byte (for directory)
;
verify_byte_r	.proc
		.logical start_zeropage
		sta cfg_stnd		;3
lp		pla			;4
		eor $0000,y		;7
		beq *+6			;9
		lda #$10		;11
		sta Status		;13
		dey			;14
		bpl lp			;16
		sta cfg_open		;19
		rts			;20
		.here
		.pend


;----------------------------------
;Print saving/searching
;
OP_save_saving	.proc
		bit msgmode
		bpl nomess
		jsr OP_cfg16up
		ldy #$51		; "SAVING "
		jsr OP_load_save_name
		jsr OP_cfg16up
		lda #32
		jsr newchrout_t
		lda #"$"
		jsr newchrout_t
		lda $c2
		jsr NO_printhx
		lda $c1
		jsr NO_printhx
		jmp OP_cfg16down
nomess		rts
		.pend

OP_load_searching_for .proc
		bit msgmode
		bpl nomess
		jsr OP_cfg16up
		ldy #$0c		; "SEARCHING "
		jsr $f12f
		ldy #$17		; "FOR "

OP_load_save_name .proc
		jsr $f12f
		ldx #0
kqu		#sta_cfg_open
		lda path_exp,x
		beq jkxe
		jsr newchrout_t
		inx
		bne kqu
jkxe		jmp OP_cfg16down
		.pend
nomess		rts
;----------------------------------
;Print loading/verifying $xxxx-$xxxx
;
OP_load_loading_end .proc
		ldx #"-"
		sec
		.byte SKIP1
OP_load_loading	.proc
		clc
		lda $ae
		sta load_autostart_start_exp
		lda $af
		sta load_autostart_start_exp+1
		bit msgmode
		bpl nomess
		jsr OP_cfg16up
		bcs bkjxd
		jsr $f5d2		;"LOADING"/"VERIFYING"
		ldx #$20
bkjxd		txa
		jsr newchrout_t
		lda #"$"
		jsr newchrout_t
		lda $af
		jsr NO_printhx
		lda $ae
		jsr NO_printhx
htnoq		jmp OP_cfg16down
		.pend

;----------------------------------
;Common start for load
;
OP_load_common	.proc
		jsr OP_save_zeropage
		jsr OP_copy_filename_to_path
		jsr OP_load_searching_for
		jmp OP_loadautos
                .pend

